﻿___TERMS_OF_SERVICE___

By creating or modifying this file you agree to Google Tag Manager's Community
Template Gallery Developer Terms of Service available at
https://developers.google.com/tag-manager/gallery-tos (or such other URL as
Google may provide), as modified from time to time.


___INFO___

{
  "type": "CLIENT",
  "id": "cvt_temp_public_id",
  "version": 1,
  "securityGroups": [],
  "displayName": "User Properties Client",
  "brand": {
    "id": "brand_dummy",
    "displayName": ""
  },
  "description": "Creates a server endpoint that can be used to request visitor data. The data included in the response is mapped using variables.",
  "containerContexts": [
    "SERVER"
  ]
}


___TEMPLATE_PARAMETERS___

[
  {
    "type": "TEXT",
    "name": "endpointPath",
    "displayName": "Endpoint Path",
    "simpleValueType": true,
    "defaultValue": "/userProperties",
    "help": "The endpoint that will be called. Requests sent to this path will be claimed by the client. The request path mustmatch this value exactly.",
    "valueValidators": [
      {
        "type": "NON_EMPTY"
      }
    ]
  },
  {
    "type": "SIMPLE_TABLE",
    "name": "allowedOrigins",
    "displayName": "Allowed Request Origins",
    "simpleTableColumns": [
      {
        "defaultValue": "",
        "displayName": "Request origin",
        "name": "origin",
        "type": "TEXT"
      }
    ],
    "alwaysInSummary": true,
    "help": "List the allowed requests origins. Leave empty to allow all."
  },
  {
    "type": "SIMPLE_TABLE",
    "name": "userProperties",
    "displayName": "User Properties",
    "simpleTableColumns": [
      {
        "defaultValue": "",
        "displayName": "Variable Input",
        "name": "property",
        "type": "TEXT",
        "valueHint": ""
      },
      {
        "defaultValue": "",
        "displayName": "dataLayer Field Name",
        "name": "keyName",
        "type": "TEXT"
      }
    ],
    "help": "Map user property values to dataLayer fields. Firestore variables can be used to retrieve data from Firestore.",
    "alwaysInSummary": true
  }
]


___SANDBOXED_JS_FOR_SERVER___

const claimRequest = require('claimRequest');
const getRequestPath = require('getRequestPath');
const logToConsole = require('logToConsole');
const getRequestHeader = require('getRequestHeader');
const JSON = require('JSON');
const setResponseStatus = require('setResponseStatus');
const setResponseBody = require('setResponseBody');
const setResponseHeader = require('setResponseHeader');
const returnResponse = require('returnResponse');

const log = msg => {
  logToConsole('User property client: ' + msg);
};

const userProperties = data.userProperties || [];

const requestPath = getRequestPath();

const requestOrigin = getRequestHeader('origin');
const allowedOrigins = data.allowedOrigins ? data.allowedOrigins.map(origin => origin.origin) : [];

// set the allowed request origin to be used in the response header
let allowedOrigin = allowedOrigins[0] || '*';
if (allowedOrigins.length > 0) {
  allowedOrigins.forEach(origin => {
    if (requestOrigin === origin) {
      allowedOrigin = requestOrigin;
    }
  });
}

// a function for setting the server response
const setResponse = (status, body, allowedOrigin) => {
  setResponseStatus(status);
  setResponseBody(JSON.stringify(body));
  setResponseHeader('access-control-allow-origin', allowedOrigin);
  setResponseHeader('access-control-allow-credentials', 'true');

  returnResponse();
};

// claim the request if the path matches the defined endoint
if (requestPath === data.endpointPath && userProperties.length > 0) {
  log('Claim user property request');
  claimRequest();

  // return a user property response if the origin matches one of the allowed origins
  if (allowedOrigin === requestOrigin || allowedOrigin === '*') {
    const properties = {};
    userProperties.forEach(property => {
      properties[property.keyName] = property.property;
    });

    // map the data into the dataLayer format
    const response = {
      event: "user_data",
      user_properties: properties
    };

    // send the dataLayer event as the response
    setResponse(200, response, requestOrigin);

  }
  
  // send an empty response
  setResponse(200, {}, allowedOrigin);
}


___SERVER_PERMISSIONS___

[
  {
    "instance": {
      "key": {
        "publicId": "read_request",
        "versionId": "1"
      },
      "param": [
        {
          "key": "headerWhitelist",
          "value": {
            "type": 2,
            "listItem": [
              {
                "type": 3,
                "mapKey": [
                  {
                    "type": 1,
                    "string": "headerName"
                  }
                ],
                "mapValue": [
                  {
                    "type": 1,
                    "string": "origin"
                  }
                ]
              }
            ]
          }
        },
        {
          "key": "queryParametersAllowed",
          "value": {
            "type": 8,
            "boolean": true
          }
        },
        {
          "key": "headersAllowed",
          "value": {
            "type": 8,
            "boolean": true
          }
        },
        {
          "key": "pathAllowed",
          "value": {
            "type": 8,
            "boolean": true
          }
        },
        {
          "key": "queryParameterAccess",
          "value": {
            "type": 1,
            "string": "any"
          }
        },
        {
          "key": "requestAccess",
          "value": {
            "type": 1,
            "string": "specific"
          }
        },
        {
          "key": "headerAccess",
          "value": {
            "type": 1,
            "string": "specific"
          }
        }
      ]
    },
    "clientAnnotations": {
      "isEditedByUser": true
    },
    "isRequired": true
  },
  {
    "instance": {
      "key": {
        "publicId": "return_response",
        "versionId": "1"
      },
      "param": []
    },
    "isRequired": true
  },
  {
    "instance": {
      "key": {
        "publicId": "logging",
        "versionId": "1"
      },
      "param": [
        {
          "key": "environments",
          "value": {
            "type": 1,
            "string": "debug"
          }
        }
      ]
    },
    "clientAnnotations": {
      "isEditedByUser": true
    },
    "isRequired": true
  },
  {
    "instance": {
      "key": {
        "publicId": "access_response",
        "versionId": "1"
      },
      "param": [
        {
          "key": "writeResponseAccess",
          "value": {
            "type": 1,
            "string": "any"
          }
        },
        {
          "key": "writeHeaderAccess",
          "value": {
            "type": 1,
            "string": "specific"
          }
        }
      ]
    },
    "clientAnnotations": {
      "isEditedByUser": true
    },
    "isRequired": true
  }
]


___TESTS___

scenarios: []


___NOTES___

Created on 10/17/2022, 10:21:31 AM


